/* Author: Derek O Reilly, Dundalk Institute of Technology, Ireland. */

class Teddy extends GameObject
{
    /* Each gameObject MUST have a constructor() and a render() method.        */
    /* If the object animates, then it must also have an updateState() method. */

    constructor(torsoCentreX, torsoCentreY, scale, torsoImage, headImage, leftArmUpperImage, leftArmLowerImage, leftArmHandImage, rightArmUpperImage, rightArmLowerImage, rightArmHandImage, leftLegUpperImage, leftLegLowerImage, leftLegFootImage, rightLegUpperImage, rightLegLowerImage, rightLegFootImage, delay = 100)
    {
        super(delay); /* as this class extends from GameObject, you must always call super() */

        this.scale = scale; // the scale is needed to convert the physical image data into the required size on the canvas

        /* These variables depend on the object */
        // assign locations in the body array for each body part
        this.HEAD = 0;
        this.TORSO = 1;
        this.LEFT_ARM_UPPER = 2;
        this.LEFT_ARM_LOWER = 3;
        this.LEFT_ARM_HAND = 4;
        this.RIGHT_ARM_UPPER = 5;
        this.RIGHT_ARM_LOWER = 6;
        this.RIGHT_ARM_HAND = 7;
        this.LEFT_LEG_UPPER = 8;
        this.LEFT_LEG_LOWER = 9;
        this.LEFT_LEG_FOOT = 10;
        this.RIGHT_LEG_UPPER = 11;
        this.RIGHT_LEG_LOWER = 12;
        this.RIGHT_LEG_FOOT = 13;
        this.NUMBER_OF_BODY_PARTS = 14;

        this.bodyPart = [];

        //  TeddyBodyPart(name, scale, sourceImage, width, height, thetha, centreOfRotationX, centreOfRotationY, hotSpotX, hotSpotY, hotSpotRadius)        
        this.bodyPart[this.TORSO] = new TeddyBodyPart(scale, torsoImage, 200, 200, 0.0, 100, 150, 100, 75, 75);
        this.bodyPart[this.HEAD] = new TeddyBodyPart(scale, headImage, 100, 100, 0.0, 50, 80, 50, 30, 40);

        this.bodyPart[this.LEFT_ARM_UPPER] = new TeddyBodyPart(scale, leftArmUpperImage, 100, 50, Math.PI / 3, 0, 25, 70, 25, 30);
        this.bodyPart[this.LEFT_ARM_LOWER] = new TeddyBodyPart(scale, leftArmLowerImage, 100, 50, Math.PI / 3, 0, 25, 70, 25, 30);
        this.bodyPart[this.LEFT_ARM_HAND] = new TeddyBodyPart(scale, leftArmHandImage, 50, 50, Math.PI / 3, 5, 35, 35, 25, 30);

        this.bodyPart[this.RIGHT_ARM_UPPER] = new TeddyBodyPart(scale, rightArmUpperImage, 100, 50, Math.PI * 2.7, 0, 25, 70, 25, 30);
        this.bodyPart[this.RIGHT_ARM_LOWER] = new TeddyBodyPart(scale, rightArmLowerImage, 100, 50, Math.PI * 2.7, 0, 25, 70, 25, 30);
        this.bodyPart[this.RIGHT_ARM_HAND] = new TeddyBodyPart(scale, rightArmHandImage, 50, 50, Math.PI * 2.7, 5, 15, 35, 15, 30);

        this.bodyPart[this.LEFT_LEG_UPPER] = new TeddyBodyPart(scale, leftLegUpperImage, 100, 100, 0.0, 50, 10, 60, 50, 40);
        this.bodyPart[this.LEFT_LEG_LOWER] = new TeddyBodyPart(scale, leftLegLowerImage, 50, 100, 0.0, 25, 10, 25, 50, 40);
        this.bodyPart[this.LEFT_LEG_FOOT] = new TeddyBodyPart(scale, leftLegFootImage, 60, 50, 0.0, 30, 5, 30, 55, 30);

        this.bodyPart[this.RIGHT_LEG_UPPER] = new TeddyBodyPart(scale, rightLegUpperImage, 100, 100, 0.0, 50, 10, 40, 50, 40);
        this.bodyPart[this.RIGHT_LEG_LOWER] = new TeddyBodyPart(scale, rightLegLowerImage, 50, 100, 0.0, 25, 10, 25, 50, 40);
        this.bodyPart[this.RIGHT_LEG_FOOT] = new TeddyBodyPart(scale, rightLegFootImage, 60, 50, 0.0, 30, 5, 30, 55, 30);

        // set the dependencies
        // torso is dependent on the canvas.
        // place the torso in the centre of the canvas
        this.bodyPart[this.TORSO].setInitialParentXandY(torsoCentreX, torsoCentreY);

        // all other body parts are dependent on the position of their parent
        // head
        this.bodyPart[this.TORSO].setChild(this.bodyPart[this.HEAD], 100, 10);

        // left arm
        this.bodyPart[this.TORSO].setChild(this.bodyPart[this.LEFT_ARM_UPPER], 180, 50);
        this.bodyPart[this.LEFT_ARM_UPPER].setChild(this.bodyPart[this.LEFT_ARM_LOWER], 80, 25);
        this.bodyPart[this.LEFT_ARM_LOWER].setChild(this.bodyPart[this.LEFT_ARM_HAND], 100, 25);

        // right arm
        this.bodyPart[this.TORSO].setChild(this.bodyPart[this.RIGHT_ARM_UPPER], 20, 50);
        this.bodyPart[this.RIGHT_ARM_UPPER].setChild(this.bodyPart[this.RIGHT_ARM_LOWER], 80, 25);
        this.bodyPart[this.RIGHT_ARM_LOWER].setChild(this.bodyPart[this.RIGHT_ARM_HAND], 100, 25);

        // left leg 
        this.bodyPart[this.TORSO].setChild(this.bodyPart[this.LEFT_LEG_UPPER], 135, 190);
        this.bodyPart[this.LEFT_LEG_UPPER].setChild(this.bodyPart[this.LEFT_LEG_LOWER], 60, 90);
        this.bodyPart[this.LEFT_LEG_LOWER].setChild(this.bodyPart[this.LEFT_LEG_FOOT], 25, 70);

        // right leg 
        this.bodyPart[this.TORSO].setChild(this.bodyPart[this.RIGHT_LEG_UPPER], 70, 190);
        this.bodyPart[this.RIGHT_LEG_UPPER].setChild(this.bodyPart[this.RIGHT_LEG_LOWER], 40, 90);
        this.bodyPart[this.RIGHT_LEG_LOWER].setChild(this.bodyPart[this.RIGHT_LEG_FOOT], 25, 70);
    }

    render()
    {
        // place all of the bodyParts in the correct position
        this.bodyPart[this.TORSO].setChildrenPositions();

        // draw all of the body parts on the main game canvas
        for (let i = this.bodyPart.length - 1; i >= 0; i--)
        {
            this.bodyPart[i].render();
        }
    }

    getBodyPart(i)
    {
        return this.bodyPart[i];
    }

    setHotSpotDrawState(state)
    {
        for (let i = 0; i < this.NUMBER_OF_BODY_PARTS; i++)
        {
            this.bodyPart[i].setDrawHotSpot(state);
        }
    }
    
    getNumberOfBodyParts()
    {
        return this.NUMBER_OF_BODY_PARTS;
    }
}