/* Author: Derek O Reilly, Dundalk Institute of Technology, Ireland.             */
/* There should always be a javaScript file with the same name as the html file. */
/* This file always holds the playGame function().                               */
/* It also holds game specific code, which will be different for each game       */




/******************** Declare game specific global data and functions *****************/
/* images must be declared as global, so that they will load before the game starts  */
let teddyBear = new Audio();
teddyBear.src = "audio/teddy_bear.m4a";

let backgroundImage = new Image();
backgroundImage.src = "images/sunshine.png";

let torsoImage = new Image();
torsoImage.src = "images/torso.png";

let headImage = new Image();
headImage.src = "images/head.png";

let leftArmUpperImage = new Image();
leftArmUpperImage.src = "images/left_arm_upper.png";

let leftArmLowerImage = new Image();
leftArmLowerImage.src = "images/left_arm_lower.png";

let leftArmHandImage = new Image();
leftArmHandImage.src = "images/left_arm_hand.png";

let rightArmUpperImage = new Image();
rightArmUpperImage.src = "images/right_arm_upper.png";

let rightArmLowerImage = new Image();
rightArmLowerImage.src = "images/right_arm_lower.png";

let rightArmHandImage = new Image();
rightArmHandImage.src = "images/right_arm_hand.png";

let leftLegUpperImage = new Image();
leftLegUpperImage.src = "images/left_leg_upper.png";

let leftLegLowerImage = new Image();
leftLegLowerImage.src = "images/left_leg_lower.png";

let leftLegFootImage = new Image();
leftLegFootImage.src = "images/left_leg_foot.png";

let rightLegUpperImage = new Image();
rightLegUpperImage.src = "images/right_leg_upper.png";

let rightLegLowerImage = new Image();
rightLegLowerImage.src = "images/right_leg_lower.png";

let rightLegFootImage = new Image();
rightLegFootImage.src = "images/right_leg_foot.png";

let oldMouseX = 0; // oldMouseX and oldMouseY hold the previous mouse position. This is needed to calculate the
let oldMouseY = 0; // direction that the mouse is moving in, so that we can rotate around a body part's centre of rotation

const START_BUTTON = 0;
const BACKGROUND = 1;
const TEDDY1 = 2;
const TEDDY2 = 3;
const TEDDY3 = 4;

let FIRST_TEDDY = TEDDY1;
let LAST_TEDDY = TEDDY3;
/******************* END OF Declare game specific data and functions *****************/







/* Always have a playGame() function                                     */
/* However, the content of this function will be different for each game */
function playGame()
{
    /* We need to initialise the game objects outside of the Game class */
    /* This function does this initialisation.                          */
    /* This function will:                                              */
    /* 1. create the various game game gameObjects                   */
    /* 2. store the game gameObjects in an array                     */
    /* 3. create a new Game to display the game gameObjects          */
    /* 4. start the Game                                                */


    /* Create the various gameObjects for this game. */
    /* This is game specific code. It will be different for each game, as each game will have it own gameObjects */
    gameObjects[BACKGROUND] = new PulsatingImage(backgroundImage, 0, 0, canvas.width, canvas.height, 20, 3, 10);
    gameObjects[TEDDY1] = new DancingTeddy(125, 120, 0.5, 5, torsoImage, headImage, leftArmUpperImage, leftArmLowerImage, leftArmHandImage, rightArmUpperImage, rightArmLowerImage, rightArmHandImage, leftLegUpperImage, leftLegLowerImage, leftLegFootImage, rightLegUpperImage, rightLegLowerImage, rightLegFootImage);
    gameObjects[TEDDY2] = new DancingTeddy(375, 120, 0.5, 30, torsoImage, headImage, leftArmUpperImage, leftArmLowerImage, leftArmHandImage, rightArmUpperImage, rightArmLowerImage, rightArmHandImage, leftLegUpperImage, leftLegLowerImage, leftLegFootImage, rightLegUpperImage, rightLegLowerImage, rightLegFootImage);
    gameObjects[TEDDY3] = new DancingTeddy(250, 330, 0.65, 2, torsoImage, headImage, leftArmUpperImage, leftArmLowerImage, leftArmHandImage, rightArmUpperImage, rightArmLowerImage, rightArmHandImage, leftLegUpperImage, leftLegLowerImage, leftLegFootImage, rightLegUpperImage, rightLegLowerImage, rightLegFootImage);

    /* END OF game specific code. */

    /* Always create a game that uses the gameObject array */
    let game = new CanvasGame();

    /* Always play the game */
    game.start();


    /* If they are needed, then include any game-specific mouse and keyboard listners */
    document.getElementById("playStopAudio").addEventListener("click", function ()
    {
        if (teddyBear.paused)
        {
            teddyBear.play();
            for (let teddy = FIRST_TEDDY; teddy <= LAST_TEDDY; teddy++)
            {
                gameObjects[teddy].start();
            }
        }
        else
        {
            teddyBear.pause();
            for (let teddy = FIRST_TEDDY; teddy <= LAST_TEDDY; teddy++)
            {
                gameObjects[teddy].stop();
            }
        }
    });
}