/* Author: Derek O Reilly, Dundalk Institute of Technology, Ireland.             */
/* There should always be a javaScript file with the same name as the html file. */
/* This file always holds the playGame function().                               */
/* It also holds game specific code, which will be different for each game       */




/******************** Declare game specific global data and functions *****************/
/* images must be declared as global, so that they will load before the game starts  */
let backgroundImage = new Image();
backgroundImage.src = "images/sunshine.png";

let torsoImage = new Image();
torsoImage.src = "images/torso.png";

let headImage = new Image();
headImage.src = "images/head.png";

let leftArmUpperImage = new Image();
leftArmUpperImage.src = "images/left_arm_upper.png";

let leftArmLowerImage = new Image();
leftArmLowerImage.src = "images/left_arm_lower.png";

let leftArmHandImage = new Image();
leftArmHandImage.src = "images/left_arm_hand.png";

let rightArmUpperImage = new Image();
rightArmUpperImage.src = "images/right_arm_upper.png";

let rightArmLowerImage = new Image();
rightArmLowerImage.src = "images/right_arm_lower.png";

let rightArmHandImage = new Image();
rightArmHandImage.src = "images/right_arm_hand.png";

let leftLegUpperImage = new Image();
leftLegUpperImage.src = "images/left_leg_upper.png";

let leftLegLowerImage = new Image();
leftLegLowerImage.src = "images/left_leg_lower.png";

let leftLegFootImage = new Image();
leftLegFootImage.src = "images/left_leg_foot.png";

let rightLegUpperImage = new Image();
rightLegUpperImage.src = "images/right_leg_upper.png";

let rightLegLowerImage = new Image();
rightLegLowerImage.src = "images/right_leg_lower.png";

let rightLegFootImage = new Image();
rightLegFootImage.src = "images/right_leg_foot.png";

let oldMouseX = 0; // oldMouseX and oldMouseY hold the previous mouse position. This is needed to calculate the
let oldMouseY = 0; // direction that the mouse is moving in, so that we can rotate around a body part's centre of rotation

const BACKGROUND = 0;
const TEDDY1 = 1;
const TEDDY2 = 2;

let FIRST_TEDDY = TEDDY1;
let LAST_TEDDY = TEDDY2;
/******************* END OF Declare game specific data and functions *****************/







/* Always have a playGame() function                                     */
/* However, the content of this function will be different for each game */
function playGame()
{
    /* We need to initialise the game objects outside of the Game class */
    /* This function does this initialisation.                          */
    /* This function will:                                              */
    /* 1. create the various game game gameObjects                   */
    /* 2. store the game gameObjects in an array                     */
    /* 3. create a new Game to display the game gameObjects          */
    /* 4. start the Game                                                */


    /* Create the various gameObjects for this game. */
    /* This is game specific code. It will be different for each game, as each game will have it own gameObjects */
    gameObjects[BACKGROUND] = new StaticImage(backgroundImage, 0, 0, canvas.width, canvas.height);
    gameObjects[TEDDY1] = new Teddy(160, 250, 0.7, torsoImage, headImage, leftArmUpperImage, leftArmLowerImage, leftArmHandImage, rightArmUpperImage, rightArmLowerImage, rightArmHandImage, leftLegUpperImage, leftLegLowerImage, leftLegFootImage, rightLegUpperImage, rightLegLowerImage, rightLegFootImage);
    gameObjects[TEDDY2] = new Teddy(400, 320, 0.4, torsoImage, headImage, leftArmUpperImage, leftArmLowerImage, leftArmHandImage, rightArmUpperImage, rightArmLowerImage, rightArmHandImage, leftLegUpperImage, leftLegLowerImage, leftLegFootImage, rightLegUpperImage, rightLegLowerImage, rightLegFootImage);

    /* show hotspots on Teddy */
    /* Teddy hotspots can be draged. This will allow us to use the mouse to move the Teddy body parts */
    for (let teddy = FIRST_TEDDY; teddy <= LAST_TEDDY; teddy++)
    {
        if (gameObjects[teddy] !== undefined)
        {
            gameObjects[teddy].setHotSpotDrawState(true);
        }
    }
    /* END OF game specific code. */

    /* Always create a game that uses the gameObject array */
    let game = new CanvasGame();

    /* Always play the game */
    game.start();

    /* If they are needed, then include any game-specific mouse and keyboard listners */
    let NO_BODYPART_SELECTED = -1;
    let selectedTeddy = TEDDY1;
    let selectedBodyPart = NO_BODYPART_SELECTED;
    document.getElementById('gameCanvas').addEventListener('mousedown', function (e)
    {
        if (e.which === 1)  // left mouse button pressed
        {
            //  mouseIsDown = true;
        }
        let canvasBoundingRectangle = document.getElementById("gameCanvas").getBoundingClientRect();
        let mouseX = e.clientX - canvasBoundingRectangle.left;
        let mouseY = e.clientY - canvasBoundingRectangle.top;

        document.body.style.cursor = "move";
        for (let i = 0; i < gameObjects[TEDDY1].getNumberOfBodyParts(); i++)
        {
            for (let teddy = FIRST_TEDDY; teddy <= LAST_TEDDY; teddy++)
            {
                if (gameObjects[teddy] !== undefined)
                {
                    if (gameObjects[teddy].getBodyPart(i).isHotSpot(mouseX, mouseY) === true)
                    {
                        selectedTeddy = teddy;
                        selectedBodyPart = i;
                    }
                }
            }
        }
        oldMouseX = mouseX;
        oldMouseY = mouseY;
    });

    let ROTATION_STEP_SIZE = 1;
    document.getElementById('gameCanvas').addEventListener('mousemove', function (e)
    {
        // only process 'mousemove' when a body part has been selected
        if (selectedBodyPart !== NO_BODYPART_SELECTED)
        {
            let canvasBoundingRectangle = document.getElementById("gameCanvas").getBoundingClientRect();
            let mouseX = e.clientX - canvasBoundingRectangle.left;
            let mouseY = e.clientY - canvasBoundingRectangle.top;

            // rotate the selected bodyPart
            gameObjects[selectedTeddy].getBodyPart(selectedBodyPart).changeRotationValue(mouseX, mouseY, ROTATION_STEP_SIZE);

            // update the x and y location
            oldMouseX = mouseX;
            oldMouseY = mouseY;
        }
    });

    document.addEventListener('mouseup', function (e)
    {
        selectedBodyPart = NO_BODYPART_SELECTED;
        document.body.style.cursor = "auto";
    });
}